"""
Boolean: True or False.

https://schema.org/Boolean
"""

from datetime import *
from copy import deepcopy
from typing import *
from time import *

from typing_extensions import TypedDict, NotRequired
from pydantic import *


from schorg.schema_org_obj import SchemaOrgObj, SchemaOrgBase


class BooleanInheritedProperties(TypedDict):
    """Boolean: True or False.

    References:
        https://schema.org/Boolean
    Note:
        Model Depth 5
    Attributes:
    """


class BooleanProperties(TypedDict):
    """Boolean: True or False.

    References:
        https://schema.org/Boolean
    Note:
        Model Depth 5
    Attributes:
    """


class BooleanAllProperties(BooleanInheritedProperties, BooleanProperties, TypedDict):
    pass


class BooleanBaseModel(SchemaOrgBase):
    id_: Optional[Any] = Field(default="Boolean", alias="@id")
    context_: Optional[Any] = Field(default=None, alias="@context")
    graph_: Optional[Any] = Field(default=None, alias="@graph")

    class Config:
        ...


def create_schema_org_model(
    type_: Union[
        BooleanProperties, BooleanInheritedProperties, BooleanAllProperties
    ] = BooleanAllProperties
) -> Type[SchemaOrgBase]:
    model = create_model_from_typeddict(type_, __base__=SchemaOrgBase)
    model.__name__ = "Boolean"
    return model


Boolean = create_schema_org_model()


def create_boolean_model(
    model: Union[BooleanProperties, BooleanInheritedProperties, BooleanAllProperties]
):
    _type = deepcopy(BooleanAllProperties)
    for k in model.__annotations__.keys():
        if k not in _type.__annotations__:
            raise TypeError(f"{k} not part of BooleanAllProperties")
    delete_keys = []
    for k in _type.__annotations__.keys():
        if k not in model.__annotations__:
            delete_keys.append(k)
    for k in delete_keys:
        del _type.__annotations__[k]
    return create_schema_org_model(type_=_type)


def schema_json(model: BooleanAllProperties):
    pydantic_type = create_boolean_model(model=model)
    return pydantic_type(model).schema_json()
