"""
An observational study design.

https://schema.org/Observational
"""

from datetime import *
from copy import deepcopy
from typing import *
from time import *

from typing_extensions import TypedDict, NotRequired
from pydantic import *


from schorg.schema_org_obj import SchemaOrgObj, SchemaOrgBase


class ObservationalInheritedProperties(TypedDict):
    """An observational study design.

    References:
        https://schema.org/Observational
    Note:
        Model Depth 6
    Attributes:
    """


class ObservationalProperties(TypedDict):
    """An observational study design.

    References:
        https://schema.org/Observational
    Note:
        Model Depth 6
    Attributes:
    """


class ObservationalAllProperties(
    ObservationalInheritedProperties, ObservationalProperties, TypedDict
):
    pass


class ObservationalBaseModel(SchemaOrgBase):
    id_: Optional[Any] = Field(default="Observational", alias="@id")
    context_: Optional[Any] = Field(default=None, alias="@context")
    graph_: Optional[Any] = Field(default=None, alias="@graph")

    class Config:
        ...


def create_schema_org_model(
    type_: Union[
        ObservationalProperties,
        ObservationalInheritedProperties,
        ObservationalAllProperties,
    ] = ObservationalAllProperties
) -> Type[SchemaOrgBase]:
    model = create_model_from_typeddict(type_, __base__=SchemaOrgBase)
    model.__name__ = "Observational"
    return model


Observational = create_schema_org_model()


def create_observational_model(
    model: Union[
        ObservationalProperties,
        ObservationalInheritedProperties,
        ObservationalAllProperties,
    ]
):
    _type = deepcopy(ObservationalAllProperties)
    for k in model.__annotations__.keys():
        if k not in _type.__annotations__:
            raise TypeError(f"{k} not part of ObservationalAllProperties")
    delete_keys = []
    for k in _type.__annotations__.keys():
        if k not in model.__annotations__:
            delete_keys.append(k)
    for k in delete_keys:
        del _type.__annotations__[k]
    return create_schema_org_model(type_=_type)


def schema_json(model: ObservationalAllProperties):
    pydantic_type = create_observational_model(model=model)
    return pydantic_type(model).schema_json()
