"""
Indicates that the item is available for pre-order.

https://schema.org/PreOrder
"""

from datetime import *
from copy import deepcopy
from typing import *
from time import *

from typing_extensions import TypedDict, NotRequired
from pydantic import *


from schorg.schema_org_obj import SchemaOrgObj, SchemaOrgBase


class PreOrderInheritedProperties(TypedDict):
    """Indicates that the item is available for pre-order.

    References:
        https://schema.org/PreOrder
    Note:
        Model Depth 5
    Attributes:
    """


class PreOrderProperties(TypedDict):
    """Indicates that the item is available for pre-order.

    References:
        https://schema.org/PreOrder
    Note:
        Model Depth 5
    Attributes:
    """


class PreOrderAllProperties(PreOrderInheritedProperties, PreOrderProperties, TypedDict):
    pass


class PreOrderBaseModel(SchemaOrgBase):
    id_: Optional[Any] = Field(default="PreOrder", alias="@id")
    context_: Optional[Any] = Field(default=None, alias="@context")
    graph_: Optional[Any] = Field(default=None, alias="@graph")

    class Config:
        ...


def create_schema_org_model(
    type_: Union[
        PreOrderProperties, PreOrderInheritedProperties, PreOrderAllProperties
    ] = PreOrderAllProperties
) -> Type[SchemaOrgBase]:
    model = create_model_from_typeddict(type_, __base__=SchemaOrgBase)
    model.__name__ = "PreOrder"
    return model


PreOrder = create_schema_org_model()


def create_preorder_model(
    model: Union[PreOrderProperties, PreOrderInheritedProperties, PreOrderAllProperties]
):
    _type = deepcopy(PreOrderAllProperties)
    for k in model.__annotations__.keys():
        if k not in _type.__annotations__:
            raise TypeError(f"{k} not part of PreOrderAllProperties")
    delete_keys = []
    for k in _type.__annotations__.keys():
        if k not in model.__annotations__:
            delete_keys.append(k)
    for k in delete_keys:
        del _type.__annotations__[k]
    return create_schema_org_model(type_=_type)


def schema_json(model: PreOrderAllProperties):
    pydantic_type = create_preorder_model(model=model)
    return pydantic_type(model).schema_json()
